use std::fmt::Display;

use crate::token::Literal;

/// Concrete value in the interpreter.
#[derive(Clone, Debug, PartialEq)]
pub enum Value {
    String(String),
    Number(f64),
    Boolean(bool),
    Nil,
}

impl Display for Value {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        match self {
            Value::String(x) => write!(f, "{x}"),
            Value::Number(x) => write!(f, "{x}"),
            Value::Boolean(x) => write!(f, "{x}"),
            Value::Nil => write!(f, "nil"),
        }
    }
}

impl Value {
    /// Return false for nil or a false boolean, true otherwise.
    pub fn is_truthy(&self) -> bool {
        match self {
            Value::Nil => false,
            Value::Boolean(b) => *b,
            _ => true,
        }
    }
}

impl From<Literal> for Value {
    fn from(value: Literal) -> Self {
        match value {
            Literal::String(x) => Value::String(x),
            Literal::Number(x) => Value::Number(x),
            Literal::Boolean(x) => Value::Boolean(x),
            Literal::Nil => Value::Nil,
        }
    }
}
